@@ -3,7 +3,7 @@ package cmd
 import (
 	"context"
 	"fmt"
-	"github.com/turbot/steampipe-plugin-sdk/v4/plugin"
+	"github.com/turbot/steampipe/pkg/cloud"
 	"log"
 	"os"
 	"runtime/debug"
@@ -14,10 +14,13 @@ import (
 	"github.com/mattn/go-isatty"
 	"github.com/spf13/cobra"
 	"github.com/spf13/viper"
+	filehelpers "github.com/turbot/go-kit/files"
 	"github.com/turbot/go-kit/helpers"
 	"github.com/turbot/steampipe-plugin-sdk/v4/logging"
+	"github.com/turbot/steampipe-plugin-sdk/v4/plugin"
 	"github.com/turbot/steampipe/pkg/cmdconfig"
 	"github.com/turbot/steampipe/pkg/constants"
+	"github.com/turbot/steampipe/pkg/error_helpers"
 	"github.com/turbot/steampipe/pkg/filepaths"
 	"github.com/turbot/steampipe/pkg/migrate"
 	"github.com/turbot/steampipe/pkg/ociinstaller/versionfile"
@@ -39,14 +42,18 @@ var rootCmd = &cobra.Command{
 		utils.LogTime("cmd.root.PersistentPreRun start")
 		defer utils.LogTime("cmd.root.PersistentPreRun end")
 
+		handleArgDeprecations()
+
 		viper.Set(constants.ConfigKeyActiveCommand, cmd)
 		viper.Set(constants.ConfigKeyActiveCommandArgs, args)
 		viper.Set(constants.ConfigKeyIsTerminalTTY, isatty.IsTerminal(os.Stdout.Fd()))
 
 		createLogger()
+
 		initGlobalConfig()
+
 		task.RunTasks()
-		// TODO enable this when we move to go 1.19
+
 		// set the max memory
 		debug.SetMemoryLimit(plugin.GetMaxMemoryBytes())
 	},
@@ -80,31 +87,32 @@ Getting started:
 func InitCmd() {
 	utils.LogTime("cmd.root.InitCmd start")
 	defer utils.LogTime("cmd.root.InitCmd end")
+	cwd, err := os.Getwd()
+	error_helpers.FailOnError(err)
 
-	rootCmd.PersistentFlags().String(constants.ArgInstallDir, filepaths.DefaultInstallDir, fmt.Sprintf("Path to the Config Directory (defaults to %s)", filepaths.DefaultInstallDir))
-	rootCmd.PersistentFlags().String(constants.ArgWorkspace, "", "Path to the workspace working directory")
-	rootCmd.PersistentFlags().String(constants.ArgWorkspaceChDir, "", "Path to the workspace working directory")
-	rootCmd.PersistentFlags().String(constants.ArgCloudHost, "cloud.steampipe.io", "Steampipe Cloud host")
-	rootCmd.PersistentFlags().String(constants.ArgCloudToken, "", "Steampipe Cloud authentication token")
-	rootCmd.PersistentFlags().String(constants.ArgWorkspaceDatabase, "local", "Steampipe Cloud workspace database")
+	rootCmd.PersistentFlags().String(constants.ArgInstallDir, filepaths.DefaultInstallDir, "Path to the Config Directory")
+	rootCmd.PersistentFlags().String(constants.ArgWorkspaceChDir, cwd, "Path to the workspace working directory")
+	rootCmd.PersistentFlags().String(constants.ArgModLocation, cwd, "Path to the workspace working directory")
 	rootCmd.PersistentFlags().Bool(constants.ArgSchemaComments, true, "Include schema comments when importing connection schemas")
+	// TODO elevate these to specific command? they are not used for plugin or mod commands
+	// or else put validation for plugin commands or at least a warning
+	rootCmd.PersistentFlags().String(constants.ArgCloudHost, constants.DefaultCloudHost, "Steampipe Cloud host")
+	rootCmd.PersistentFlags().String(constants.ArgCloudToken, "", "Steampipe Cloud authentication token")
+	rootCmd.PersistentFlags().String(constants.ArgWorkspaceDatabase, constants.DefaultWorkspaceDatabase, "Steampipe Cloud workspace database")
+	rootCmd.PersistentFlags().String(constants.ArgWorkspaceProfile, "default", "The workspace profile to use")
 
-	rootCmd.Flag(constants.ArgWorkspace).Deprecated = "please use workspace-chdir"
+	// deprecate ArgWorkspaceChDir
+	workspaceChDirFlag := rootCmd.PersistentFlags().Lookup(constants.ArgWorkspaceChDir)
+	workspaceChDirFlag.Deprecated = "use --mod-location"
 
-	err := viper.BindPFlag(constants.ArgInstallDir, rootCmd.PersistentFlags().Lookup(constants.ArgInstallDir))
-	utils.FailOnError(err)
-	err = viper.BindPFlag(constants.ArgWorkspace, rootCmd.PersistentFlags().Lookup(constants.ArgWorkspace))
-	utils.FailOnError(err)
-	err = viper.BindPFlag(constants.ArgWorkspaceChDir, rootCmd.PersistentFlags().Lookup(constants.ArgWorkspaceChDir))
-	utils.FailOnError(err)
-	err = viper.BindPFlag(constants.ArgCloudHost, rootCmd.PersistentFlags().Lookup(constants.ArgCloudHost))
-	utils.FailOnError(err)
-	err = viper.BindPFlag(constants.ArgCloudToken, rootCmd.PersistentFlags().Lookup(constants.ArgCloudToken))
-	utils.FailOnError(err)
-	err = viper.BindPFlag(constants.ArgWorkspaceDatabase, rootCmd.PersistentFlags().Lookup(constants.ArgWorkspaceDatabase))
-	utils.FailOnError(err)
-	err = viper.BindPFlag(constants.ArgSchemaComments, rootCmd.PersistentFlags().Lookup(constants.ArgSchemaComments))
-	utils.FailOnError(err)
+	error_helpers.FailOnError(viper.BindPFlag(constants.ArgInstallDir, rootCmd.PersistentFlags().Lookup(constants.ArgInstallDir)))
+	error_helpers.FailOnError(viper.BindPFlag(constants.ArgWorkspaceChDir, workspaceChDirFlag))
+	error_helpers.FailOnError(viper.BindPFlag(constants.ArgModLocation, rootCmd.PersistentFlags().Lookup(constants.ArgModLocation)))
+	error_helpers.FailOnError(viper.BindPFlag(constants.ArgCloudHost, rootCmd.PersistentFlags().Lookup(constants.ArgCloudHost)))
+	error_helpers.FailOnError(viper.BindPFlag(constants.ArgCloudToken, rootCmd.PersistentFlags().Lookup(constants.ArgCloudToken)))
+	error_helpers.FailOnError(viper.BindPFlag(constants.ArgWorkspaceDatabase, rootCmd.PersistentFlags().Lookup(constants.ArgWorkspaceDatabase)))
+	error_helpers.FailOnError(viper.BindPFlag(constants.ArgSchemaComments, rootCmd.PersistentFlags().Lookup(constants.ArgSchemaComments)))
+	error_helpers.FailOnError(viper.BindPFlag(constants.ArgWorkspaceProfile, rootCmd.PersistentFlags().Lookup(constants.ArgWorkspaceProfile)))
 
 	AddCommands()
 
@@ -121,6 +129,12 @@ func InitCmd() {
 
 }
 
+func handleArgDeprecations() {
+	if !viper.IsSet(constants.ArgModLocation) && viper.IsSet(constants.ArgWorkspaceChDir) {
+		viper.Set(constants.ArgModLocation, viper.GetString(constants.ArgWorkspaceChDir))
+	}
+}
+
 func hideRootFlags(flags ...string) {
 	for _, flag := range flags {
 		rootCmd.Flag(flag).Hidden = true
@@ -132,44 +146,116 @@ func initGlobalConfig() {
 	utils.LogTime("cmd.root.initGlobalConfig start")
 	defer utils.LogTime("cmd.root.initGlobalConfig end")
 
-	// setup viper with the essential path config (workspace-chdir and install-dir)
-	cmdconfig.BootstrapViper()
+	// load workspace profile from the configured install dir
+	loader, err := loadWorkspaceProfile()
+	error_helpers.FailOnError(err)
 
-	// set the working folder
-	workspaceChdir := setWorkspaceChDir()
+	// set global workspace profile
+	steampipeconfig.GlobalWorkspaceProfile = loader.GetActiveWorkspaceProfile()
 
-	// set global containing install dir
-	setInstallDir()
+	// set-up viper with defaults from the env and default workspace profile
+	err = cmdconfig.BootstrapViper(loader)
+	error_helpers.FailOnError(err)
 
-	var cmd = viper.Get(constants.ConfigKeyActiveCommand).(*cobra.Command)
+	// set global containing the configured install dir (create directory if needed)
+	ensureInstallDir(viper.GetString(constants.ArgInstallDir))
 
-	// migrate all legacy config files to use snake casing (migrated in v0.14.0)
-	err := migrateLegacyFiles()
-	utils.FailOnErrorWithMessage(err, "failed to migrate steampipe data files")
-
-	// load config (this sets the global config steampipeconfig.Config)
-	config, err := steampipeconfig.LoadSteampipeConfig(workspaceChdir, cmd.Name())
-	utils.FailOnError(err)
+	// load the connection config and HCL options
+	var cmdName = viper.Get(constants.ConfigKeyActiveCommand).(*cobra.Command).Name()
+	config, err := steampipeconfig.LoadSteampipeConfig(viper.GetString(constants.ArgModLocation), cmdName)
+	error_helpers.FailOnError(err)
 
+	// store global config
 	steampipeconfig.GlobalConfig = config
 
-	// set viper config defaults from config and env vars
-	cmdconfig.SetViperDefaults(steampipeconfig.GlobalConfig.ConfigMap())
+	// set viper defaults from this config
+	cmdconfig.SetDefaultsFromConfig(steampipeconfig.GlobalConfig.ConfigMap())
+
+	// if an explicit workspace profile was set, add to viper as highest precedence default
+	// NOTE: if install_dir/mod_location are set these will already have been passed to viper by BootstrapViper
+	if loader.ConfiguredProfile != nil {
+		cmdconfig.SetDefaultsFromConfig(loader.ConfiguredProfile.ConfigMap())
+	}
+
+	// NOTE: we need to resolve the token separately
+	// - that is because we need the resolved value of ArgCloudHost in order to load any saved token
+	// and we cannot get this until the other config has been resolved
+	err = setCloudTokenDefault(loader)
+	error_helpers.FailOnError(err)
 
 	// now validate all config values have appropriate values
 	err = validateConfig()
-	utils.FailOnErrorWithMessage(err, "failed to validate config")
+	error_helpers.FailOnErrorWithMessage(err, "failed to validate config")
+
+	// migrate all legacy config files to use snake casing (migrated in v0.14.0)
+	err = migrateLegacyFiles()
+	error_helpers.FailOnErrorWithMessage(err, "failed to migrate steampipe data files")
+}
+
+func setCloudTokenDefault(loader *steampipeconfig.WorkspaceProfileLoader) error {
+	/*
+	   saved cloud token
+	   cloud_token in default workspace
+	   explicit env var (STEAMIPE_CLOUD_TOKEN ) wins over
+	   cloud_token in specific workspace
+	*/
+	// set viper defaults in order of increasing precedence
+	// 1) saved cloud token
+	savedToken, err := cloud.LoadToken()
+	if err != nil {
+		return err
+	}
+	if savedToken != "" {
+		viper.SetDefault(constants.ArgCloudToken, savedToken)
+	}
+	// 2) default profile cloud token
+	if loader.DefaultProfile.CloudToken != nil {
+		viper.SetDefault(constants.ArgCloudToken, *loader.DefaultProfile.CloudToken)
+	}
+	// 3) env var (STEAMIPE_CLOUD_TOKEN )
+	cmdconfig.SetDefaultFromEnv(constants.EnvCloudToken, constants.ArgCloudToken, "string")
+
+	// 4) explicit workspace profile
+	if p := loader.ConfiguredProfile; p != nil && p.CloudToken != nil {
+		viper.SetDefault(constants.ArgCloudToken, *p.CloudToken)
+	}
+	return nil
+}
+
+func loadWorkspaceProfile() (*steampipeconfig.WorkspaceProfileLoader, error) {
+	// set viper default for workspace profile, using STEAMPIPE_WORKSPACE env var
+	cmdconfig.SetDefaultFromEnv(constants.EnvWorkspaceProfile, constants.ArgWorkspaceProfile, "string")
+	// set viper default for install dir, using STEAMPIPE_INSTALL_DIR env var
+	cmdconfig.SetDefaultFromEnv(constants.EnvInstallDir, constants.ArgInstallDir, "string")
+
+	// resolve the workspace profile dir
+	installDir, err := filehelpers.Tildefy(viper.GetString(constants.ArgInstallDir))
+	if err != nil {
+		return nil, err
+	}
+
+	workspaceProfileDir, err := filepaths.WorkspaceProfileDir(installDir)
+	if err != nil {
+		return nil, err
+	}
+
+	// create loader
+	loader, err := steampipeconfig.NewWorkspaceProfileLoader(workspaceProfileDir)
+	if err != nil {
+		return nil, err
+	}
+
+	return loader, nil
 }
 
 // migrate all data files to use snake casing for property names
 func migrateLegacyFiles() error {
-
 	// skip migration for plugin manager commands because the plugin-manager will have
 	// been started by some other steampipe command, which would have done the migration already
 	if viper.Get(constants.ConfigKeyActiveCommand).(*cobra.Command).Name() == "plugin-manager" {
 		return nil
 	}
-	return utils.CombineErrors(
+	return error_helpers.CombineErrors(
 		migrate.Migrate(&statefile.State{}, filepaths.LegacyStateFilePath()),
 		migrate.Migrate(&steampipeconfig.ConnectionDataMap{}, filepaths.ConnectionStatePath()),
 		migrate.Migrate(&versionfile.PluginVersionFile{}, filepaths.PluginVersionFilePath()),
@@ -178,6 +264,7 @@ func migrateLegacyFiles() error {
 }
 
 // now validate  config values have appropriate values
+// (currently validates telemetry)
 func validateConfig() error {
 	telemetry := viper.GetString(constants.ArgTelemetry)
 	if !helpers.StringSliceContains(constants.TelemetryLevels, telemetry) {
@@ -186,22 +273,6 @@ func validateConfig() error {
 	return nil
 }
 
-func setWorkspaceChDir() string {
-	workspaceChdir := viper.GetString(constants.ArgWorkspaceChDir)
-	workspace := viper.GetString(constants.ArgWorkspace)
-	if workspace != "" {
-		workspaceChdir = workspace
-
-	}
-	if workspaceChdir == "" {
-		cwd, err := os.Getwd()
-		utils.FailOnError(err)
-		workspaceChdir = cwd
-	}
-	viper.Set(constants.ArgWorkspaceChDir, workspaceChdir)
-	return workspaceChdir
-}
-
 // create a hclog logger with the level specified by the SP_LOG env var
 func createLogger() {
 	level := logging.LogLevel()
@@ -221,17 +292,15 @@ func createLogger() {
 	log.SetFlags(0)
 }
 
-// set the top level ~/.steampipe folder (creates if it doesnt exist)
-func setInstallDir() {
-	utils.LogTime("cmd.root.setInstallDir start")
-	defer utils.LogTime("cmd.root.setInstallDir end")
-
-	installDir, err := helpers.Tildefy(viper.GetString(constants.ArgInstallDir))
-	utils.FailOnErrorWithMessage(err, "failed to sanitize install directory")
+func ensureInstallDir(installDir string) {
+	log.Printf("[TRACE] ensureInstallDir %s", installDir)
 	if _, err := os.Stat(installDir); os.IsNotExist(err) {
+		log.Printf("[TRACE] creating install dir")
 		err = os.MkdirAll(installDir, 0755)
-		utils.FailOnErrorWithMessage(err, fmt.Sprintf("could not create installation directory: %s", installDir))
+		error_helpers.FailOnErrorWithMessage(err, fmt.Sprintf("could not create installation directory: %s", installDir))
 	}
+
+	// store as SteampipeDir
 	filepaths.SteampipeDir = installDir
 }
 
@@ -247,6 +316,7 @@ func AddCommands() {
 		pluginManagerCmd(),
 		dashboardCmd(),
 		variableCmd(),
+		loginCmd(),
 	)
 }
 
