@@ -20,18 +20,19 @@ const (
 var DatabaseListenAddresses = []string{"localhost", "127.0.0.1"}
 
 const (
-	DatabaseDefaultPort   = 9193
-	DatabaseSuperUser     = "root"
-	DatabaseUser          = "steampipe"
-	DatabaseName          = "steampipe"
-	DatabaseUsersRole     = "steampipe_users"
-	DefaultMaxConnections = 5
+	DatabaseDefaultPort              = 9193
+	DatabaseDefaultCheckQueryTimeout = 240
+	DatabaseSuperUser                = "root"
+	DatabaseUser                     = "steampipe"
+	DatabaseName                     = "steampipe"
+	DatabaseUsersRole                = "steampipe_users"
+	DefaultMaxConnections            = 5
 )
 
 // constants for installing db and fdw images
 const (
 	DatabaseVersion = "14.2.0"
-	FdwVersion      = "1.4.2"
+	FdwVersion      = "1.4.4"
 
 	// PostgresImageRef is the OCI Image ref for the database binaries
 	PostgresImageRef    = "us-docker.pkg.dev/steampipe/steampipe/db:14.2.0"
@@ -93,6 +94,7 @@ const (
 	IntrospectionTableDashboardCard      = "steampipe_dashboard_card"
 	IntrospectionTableDashboardChart     = "steampipe_dashboard_chart"
 	IntrospectionTableDashboardFlow      = "steampipe_dashboard_flow"
+	IntrospectionTableDashboardGraph     = "steampipe_dashboard_graph"
 	IntrospectionTableDashboardHierarchy = "steampipe_dashboard_hierarchy"
 	IntrospectionTableDashboardImage     = "steampipe_dashboard_image"
 	IntrospectionTableDashboardInput     = "steampipe_dashboard_input"
