@@ -442,6 +442,7 @@ func (r *ControlRun) setRunStatus(ctx context.Context, status controlstatus.Cont
 	r.stateLock.Unlock()
 
 	if r.Finished() {
-		r.doneChan <- true
+		// close the doneChan - we don't need it anymore
+		close(r.doneChan)
 	}
 }
