@@ -5,9 +5,9 @@ import (
 
 	"github.com/turbot/steampipe/pkg/constants"
 	"github.com/turbot/steampipe/pkg/db/db_local"
+	"github.com/turbot/steampipe/pkg/error_helpers"
 	"github.com/turbot/steampipe/pkg/query"
 	"github.com/turbot/steampipe/pkg/query/queryresult"
-	"github.com/turbot/steampipe/pkg/utils"
 )
 
 // RunInteractivePrompt starts the interactive query prompt
@@ -16,7 +16,7 @@ func RunInteractivePrompt(ctx context.Context, initData *query.InitData) (*query
 
 	interactiveClient, err := newInteractiveClient(ctx, initData, resultsStreamer)
 	if err != nil {
-		utils.ShowErrorWithMessage(ctx, err, "interactive client failed to initialize")
+		error_helpers.ShowErrorWithMessage(ctx, err, "interactive client failed to initialize")
 		// do not bind shutdown to any cancellable context
 		db_local.ShutdownService(ctx, constants.InvokerQuery)
 		return nil, err
