@@ -38,7 +38,7 @@ func init() {
 				FullName:  "mod.local",
 				Require:   require,
 				Title:     toStringPointer("no_mod_sql_files"),
-				ResourceMaps: &modconfig.ModResources{
+				ResourceMaps: &modconfig.ResourceMaps{
 					Queries: map[string]*modconfig.Query{
 						"local.query.q1": {
 							ShortName: "q1",
@@ -61,7 +61,7 @@ func init() {
 				Title:     toStringPointer("no_mod_hcl_queries"),
 				FullName:  "mod.local",
 				Require:   require,
-				ResourceMaps: &modconfig.ModResources{
+				ResourceMaps: &modconfig.ResourceMaps{
 					Queries: map[string]*modconfig.Query{
 						"local.query.q1": {
 							ShortName:   "q1",
@@ -93,7 +93,7 @@ func init() {
 				Require:      require,
 				Title:        toStringPointer("M1"),
 				Description:  toStringPointer("THIS IS M1"),
-				ResourceMaps: &modconfig.ModResources{},
+				ResourceMaps: &modconfig.ResourceMaps{},
 			},
 		},
 		"single_mod_one_query": {
@@ -104,7 +104,7 @@ func init() {
 				Require:     require,
 				Title:       toStringPointer("M1"),
 				Description: toStringPointer("THIS IS M1"),
-				ResourceMaps: &modconfig.ModResources{
+				ResourceMaps: &modconfig.ResourceMaps{
 					Queries: map[string]*modconfig.Query{
 						"m1.query.q1": {
 							ShortName:   "q1",
@@ -125,7 +125,7 @@ func init() {
 				Require:     require,
 				Title:       toStringPointer("M1"),
 				Description: toStringPointer("THIS IS M1"),
-				ResourceMaps: &modconfig.ModResources{
+				ResourceMaps: &modconfig.ResourceMaps{
 					Queries: map[string]*modconfig.Query{
 						"m1.query.q1": {
 							ShortName:   "q1",
@@ -156,7 +156,7 @@ func init() {
 		// 		Require:     require,
 		// 		Title:       toStringPointer("M1"),
 		// 		Description: toStringPointer("THIS IS M1"),
-		// 		ResourceMaps: &modconfig.ModResources{
+		// 		ResourceMaps: &modconfig.ResourceMaps{
 		// 			Queries: map[string]*modconfig.Query{
 		// 				"m1.query.q1": {
 		// 					ShortName:   "q1",
@@ -215,7 +215,7 @@ func init() {
 				Require:     require,
 				Title:       toStringPointer("M1"),
 				Description: toStringPointer("THIS IS M1"),
-				ResourceMaps: &modconfig.ModResources{
+				ResourceMaps: &modconfig.ResourceMaps{
 					Queries: map[string]*modconfig.Query{
 						"m1.query.q1": {
 							ShortName:   "q1",
@@ -246,7 +246,7 @@ func init() {
 				Require:     require,
 				Title:       toStringPointer("M1"),
 				Description: toStringPointer("THIS IS M1"),
-				ResourceMaps: &modconfig.ModResources{
+				ResourceMaps: &modconfig.ResourceMaps{
 					Queries: map[string]*modconfig.Query{
 						"m1.query.q1": {
 							ShortName:       "q1",
@@ -350,7 +350,7 @@ func init() {
 				Require:     require,
 				Title:       toStringPointer("M1"),
 				Description: toStringPointer("THIS IS M1"),
-				ResourceMaps: &modconfig.ModResources{
+				ResourceMaps: &modconfig.ResourceMaps{
 					Queries: map[string]*modconfig.Query{"m1.query.q1": {ShortName: "q1", FullName: "m1.query.q1",
 						SQL: toStringPointer("select 1")}},
 				},
@@ -364,7 +364,7 @@ func init() {
 				Require:     require,
 				Title:       toStringPointer("M1"),
 				Description: toStringPointer("THIS IS M1"),
-				ResourceMaps: &modconfig.ModResources{
+				ResourceMaps: &modconfig.ResourceMaps{
 					Queries: map[string]*modconfig.Query{
 						"m1.query.q1": {
 							ShortName:   "q1",
@@ -394,7 +394,7 @@ func init() {
 				Require:     require,
 				Title:       toStringPointer("M1"),
 				Description: toStringPointer("THIS IS M1"),
-				ResourceMaps: &modconfig.ModResources{
+				ResourceMaps: &modconfig.ResourceMaps{
 					Queries: map[string]*modconfig.Query{
 						"m1.query.q1": {
 							ShortName:   "q1",
@@ -422,7 +422,7 @@ func init() {
 				Require:     require,
 				Title:       toStringPointer("M1"),
 				Description: toStringPointer("THIS IS M1"),
-				ResourceMaps: &modconfig.ModResources{
+				ResourceMaps: &modconfig.ResourceMaps{
 					Queries: map[string]*modconfig.Query{
 						"m1.query.q1": {
 							ShortName:   "q1",
@@ -450,7 +450,7 @@ func init() {
 				Require:     require,
 				Title:       toStringPointer("M1"),
 				Description: toStringPointer("THIS IS M1"),
-				ResourceMaps: &modconfig.ModResources{
+				ResourceMaps: &modconfig.ResourceMaps{
 					Queries: map[string]*modconfig.Query{
 						"m1.query.q1": {
 							ShortName: "q1",
@@ -474,14 +474,14 @@ func init() {
 				Require:     require,
 				Title:       toStringPointer("simple report"),
 				Description: toStringPointer("this mod contains a simple report"),
-				ResourceMaps: &modconfig.ModResources{
+				ResourceMaps: &modconfig.ResourceMaps{
 					Dashboards: map[string]*modconfig.Dashboard{
 						"simple_report.dashboard.simple_report": {
 							ShortName:       "simple_report",
 							FullName:        "simple_report.dashboard.simple_report",
 							UnqualifiedName: "dashboard.simple_report",
 							ChildNames:      []string{"simple_report.text.dashboard_simple_report_anonymous_text_0", "simple_report.chart.dashboard_simple_report_anonymous_chart_0"},
-							HclType:         "dashboard",
+							//HclType:         "dashboard",
 						},
 					},
 					DashboardCharts: map[string]*modconfig.DashboardChart{
@@ -512,14 +512,14 @@ func init() {
 				Require:     require,
 				Title:       toStringPointer("simple report with container"),
 				Description: toStringPointer("this mod contains a simple report with containers"),
-				ResourceMaps: &modconfig.ModResources{
+				ResourceMaps: &modconfig.ResourceMaps{
 					Dashboards: map[string]*modconfig.Dashboard{
 						"simple_container_report.dashboard.simple_container_report": {
 							ShortName:       "simple_container_report",
 							FullName:        "simple_container_report.dashboard.simple_container_report",
 							UnqualifiedName: "dashboard.simple_container_report",
 							ChildNames:      []string{"simple_container_report.container.dashboard_simple_container_report_anonymous_container_0"},
-							HclType:         "dashboard",
+							//HclType:         "dashboard",
 						},
 					},
 					DashboardContainers: map[string]*modconfig.DashboardContainer{
@@ -558,14 +558,14 @@ func init() {
 				Require:     require,
 				Title:       toStringPointer("report with multiple sibling containers"),
 				Description: toStringPointer("this mod contains a report with multiple sibling containers"),
-				ResourceMaps: &modconfig.ModResources{
+				ResourceMaps: &modconfig.ResourceMaps{
 					Dashboards: map[string]*modconfig.Dashboard{
 						"sibling_containers_report.dashboard.sibling_containers_report": {
 							ShortName:       "sibling_containers_report",
 							FullName:        "sibling_containers_report.dashboard.sibling_containers_report",
 							UnqualifiedName: "dashboard.sibling_containers_report",
 							ChildNames:      []string{"sibling_containers_report.container.dashboard_sibling_containers_report_anonymous_container_0", "sibling_containers_report.container.dashboard_sibling_containers_report_anonymous_container_1", "sibling_containers_report.container.dashboard_sibling_containers_report_anonymous_container_2"},
-							HclType:         "dashboard",
+							//HclType:         "dashboard",
 						},
 					},
 					DashboardContainers: map[string]*modconfig.DashboardContainer{
@@ -642,14 +642,14 @@ func init() {
 				Require:     require,
 				Title:       toStringPointer("report with nested containers"),
 				Description: toStringPointer("this mod contains a report with nested containers"),
-				ResourceMaps: &modconfig.ModResources{
+				ResourceMaps: &modconfig.ResourceMaps{
 					Dashboards: map[string]*modconfig.Dashboard{
 						"nested_containers_report.dashboard.nested_containers_report": {
 							ShortName:       "nested_containers_report",
 							FullName:        "nested_containers_report.dashboard.nested_containers_report",
 							UnqualifiedName: "dashboard.nested_containers_report",
 							ChildNames:      []string{"nested_containers_report.container.dashboard_nested_containers_report_anonymous_container_0"},
-							HclType:         "dashboard",
+							//HclType:         "dashboard",
 						},
 					},
 					DashboardContainers: map[string]*modconfig.DashboardContainer{
@@ -738,7 +738,7 @@ func init() {
 				Require:     require,
 				Title:       toStringPointer("report with axes"),
 				Description: toStringPointer("This mod tests base values overriding functionality"),
-				ResourceMaps: &modconfig.ModResources{
+				ResourceMaps: &modconfig.ResourceMaps{
 					Dashboards: map[string]*modconfig.Dashboard{
 						"report_axes.dashboard.override_base_values": {
 							ShortName:       "override_base_values",
@@ -746,7 +746,7 @@ func init() {
 							UnqualifiedName: "dashboard.override_base_values",
 							Title:           toStringPointer("override_base_values"),
 							ChildNames:      []string{"report_axes.chart.dashboard_override_base_values_anonymous_chart_0"},
-							HclType:         "dashboard",
+							//HclType:         "dashboard",
 						},
 					},
 					DashboardCharts: map[string]*modconfig.DashboardChart{
@@ -810,7 +810,7 @@ func init() {
 				Require:     require,
 				Description: toStringPointer("This mod tests inheriting from base functionality"),
 				Title:       toStringPointer("report base 1"),
-				ResourceMaps: &modconfig.ModResources{
+				ResourceMaps: &modconfig.ResourceMaps{
 					Queries: map[string]*modconfig.Query{
 						"report_base1.query.basic_query": {
 							ShortName:       "basic_query",
@@ -826,7 +826,7 @@ func init() {
 							UnqualifiedName: "dashboard.inheriting_from_base",
 							Title:           toStringPointer("inheriting_from_base"),
 							ChildNames:      []string{"report_base1.chart.dashboard_inheriting_from_base_anonymous_chart_0"},
-							HclType:         "dashboard",
+							//HclType:         "dashboard",
 						},
 					},
 					DashboardCharts: map[string]*modconfig.DashboardChart{
@@ -893,7 +893,7 @@ func init() {
 				Require:     require,
 				Description: toStringPointer("This mod contains a dashboard with a container with all possible child resources"),
 				Title:       toStringPointer("container with all possible child resources"),
-				ResourceMaps: &modconfig.ModResources{
+				ResourceMaps: &modconfig.ResourceMaps{
 					Dashboards: map[string]*modconfig.Dashboard{
 						"container_with_children.dashboard.container_with_child_res": {
 							ShortName:       "container_with_child_res",
@@ -901,7 +901,7 @@ func init() {
 							UnqualifiedName: "dashboard.container_with_child_res",
 							Title:           toStringPointer("container with child resources"),
 							ChildNames:      []string{"container_with_children.container.dashboard_container_with_child_res_anonymous_container_0"},
-							HclType:         "dashboard",
+							//HclType:         "dashboard",
 						},
 					},
 					DashboardContainers: map[string]*modconfig.DashboardContainer{
@@ -910,7 +910,15 @@ func init() {
 							FullName:        "container_with_children.container.dashboard_container_with_child_res_anonymous_container_0",
 							UnqualifiedName: "container.dashboard_container_with_child_res_anonymous_container_0",
 							Title:           toStringPointer("example container with all possible child resources"),
-							ChildNames:      []string{"container_with_children.chart.container_dashboard_container_with_child_res_anonymous_container_0_anonymous_chart_0", "container_with_children.card.container_dashboard_container_with_child_res_anonymous_container_0_anonymous_card_0", "container_with_children.flow.container_dashboard_container_with_child_res_anonymous_container_0_anonymous_flow_0", "container_with_children.hierarchy.container_dashboard_container_with_child_res_anonymous_container_0_anonymous_hierarchy_0", "container_with_children.image.container_dashboard_container_with_child_res_anonymous_container_0_anonymous_image_0", "container_with_children.table.container_dashboard_container_with_child_res_anonymous_container_0_anonymous_table_0", "container_with_children.text.container_dashboard_container_with_child_res_anonymous_container_0_anonymous_text_0"},
+							ChildNames: []string{
+								"container_with_children.chart.container_dashboard_container_with_child_res_anonymous_container_0_anonymous_chart_0",
+								"container_with_children.card.container_dashboard_container_with_child_res_anonymous_container_0_anonymous_card_0",
+								"container_with_children.flow.container_dashboard_container_with_child_res_anonymous_container_0_anonymous_flow_0",
+								"container_with_children.graph.container_dashboard_container_with_child_res_anonymous_container_0_anonymous_graph_0",
+								"container_with_children.hierarchy.container_dashboard_container_with_child_res_anonymous_container_0_anonymous_hierarchy_0",
+								"container_with_children.image.container_dashboard_container_with_child_res_anonymous_container_0_anonymous_image_0",
+								"container_with_children.table.container_dashboard_container_with_child_res_anonymous_container_0_anonymous_table_0",
+								"container_with_children.text.container_dashboard_container_with_child_res_anonymous_container_0_anonymous_text_0"},
 						},
 					},
 					DashboardCards: map[string]*modconfig.DashboardCard{
@@ -941,6 +949,15 @@ func init() {
 							Type:            toStringPointer("sankey"),
 						},
 					},
+					DashboardGraphs: map[string]*modconfig.DashboardGraph{
+						"container_with_children.graph.container_dashboard_container_with_child_res_anonymous_container_0_anonymous_graph_0": {
+							FullName:        "container_with_children.graph.container_dashboard_container_with_child_res_anonymous_container_0_anonymous_graph_0",
+							ShortName:       "container_dashboard_container_with_child_res_anonymous_container_0_anonymous_graph_0",
+							UnqualifiedName: "graph.container_dashboard_container_with_child_res_anonymous_container_0_anonymous_graph_0",
+							Title:           toStringPointer("example graph"),
+							Type:            toStringPointer("graph"),
+						},
+					},
 					DashboardHierarchies: map[string]*modconfig.DashboardHierarchy{
 						"container_with_children.hierarchy.container_dashboard_container_with_child_res_anonymous_container_0_anonymous_hierarchy_0": {
 							FullName:        "container_with_children.hierarchy.container_dashboard_container_with_child_res_anonymous_container_0_anonymous_hierarchy_0",
@@ -988,15 +1005,25 @@ func init() {
 				Require:     require,
 				Description: toStringPointer("This mod contains a dashboard with all possible child resources"),
 				Title:       toStringPointer("dashboard with all possible child resources"),
-				ResourceMaps: &modconfig.ModResources{
+				ResourceMaps: &modconfig.ResourceMaps{
 					Dashboards: map[string]*modconfig.Dashboard{
 						"dashboard_with_children.dashboard.dashboard_with_child_res": {
 							ShortName:       "dashboard_with_child_res",
 							FullName:        "dashboard_with_children.dashboard.dashboard_with_child_res",
 							UnqualifiedName: "dashboard.dashboard_with_child_res",
 							Title:           toStringPointer("dashboard with child resources"),
-							ChildNames:      []string{"dashboard_with_children.container.dashboard_dashboard_with_child_res_anonymous_container_0", "dashboard_with_children.chart.dashboard_dashboard_with_child_res_anonymous_chart_0", "dashboard_with_children.card.dashboard_dashboard_with_child_res_anonymous_card_0", "dashboard_with_children.flow.dashboard_dashboard_with_child_res_anonymous_flow_0", "dashboard_with_children.hierarchy.dashboard_dashboard_with_child_res_anonymous_hierarchy_0", "dashboard_with_children.image.dashboard_dashboard_with_child_res_anonymous_image_0", "dashboard_with_children.input.i1", "dashboard_with_children.table.dashboard_dashboard_with_child_res_anonymous_table_0", "dashboard_with_children.text.dashboard_dashboard_with_child_res_anonymous_text_0"},
-							HclType:         "dashboard",
+							ChildNames: []string{
+								"dashboard_with_children.container.dashboard_dashboard_with_child_res_anonymous_container_0",
+								"dashboard_with_children.chart.dashboard_dashboard_with_child_res_anonymous_chart_0",
+								"dashboard_with_children.card.dashboard_dashboard_with_child_res_anonymous_card_0",
+								"dashboard_with_children.flow.dashboard_dashboard_with_child_res_anonymous_flow_0",
+								"dashboard_with_children.graph.dashboard_dashboard_with_child_res_anonymous_graph_0",
+								"dashboard_with_children.hierarchy.dashboard_dashboard_with_child_res_anonymous_hierarchy_0",
+								"dashboard_with_children.image.dashboard_dashboard_with_child_res_anonymous_image_0",
+								"dashboard_with_children.input.i1",
+								"dashboard_with_children.table.dashboard_dashboard_with_child_res_anonymous_table_0",
+								"dashboard_with_children.text.dashboard_dashboard_with_child_res_anonymous_text_0"},
+							//HclType: "dashboard",
 						},
 					},
 					DashboardContainers: map[string]*modconfig.DashboardContainer{
@@ -1035,6 +1062,15 @@ func init() {
 							Type:            toStringPointer("sankey"),
 						},
 					},
+					DashboardGraphs: map[string]*modconfig.DashboardGraph{
+						"dashboard_with_children.graph.dashboard_dashboard_with_child_res_anonymous_graph_0": {
+							FullName:        "dashboard_with_children.graph.dashboard_dashboard_with_child_res_anonymous_graph_0",
+							ShortName:       "dashboard_dashboard_with_child_res_anonymous_graph_0",
+							UnqualifiedName: "graph.dashboard_dashboard_with_child_res_anonymous_graph_0",
+							Title:           toStringPointer("example graph"),
+							Type:            toStringPointer("graph"),
+						},
+					},
 					DashboardHierarchies: map[string]*modconfig.DashboardHierarchy{
 						"dashboard_with_children.hierarchy.dashboard_dashboard_with_child_res_anonymous_hierarchy_0": {
 							FullName:        "dashboard_with_children.hierarchy.dashboard_dashboard_with_child_res_anonymous_hierarchy_0",
@@ -1092,14 +1128,14 @@ func init() {
 				Require:     require,
 				Description: toStringPointer("this mod is to test the resource naming"),
 				Title:       toStringPointer("dashboard resource naming"),
-				ResourceMaps: &modconfig.ModResources{
+				ResourceMaps: &modconfig.ResourceMaps{
 					Dashboards: map[string]*modconfig.Dashboard{
 						"dashboard_resource_naming.dashboard.anonymous_naming": {
 							FullName:        "dashboard_resource_naming.dashboard.anonymous_naming",
 							ShortName:       "anonymous_naming",
 							UnqualifiedName: "dashboard.anonymous_naming",
 							ChildNames:      []string{"dashboard_resource_naming.chart.dashboard_anonymous_naming_anonymous_chart_0", "dashboard_resource_naming.container.dashboard_anonymous_naming_anonymous_container_0", "dashboard_resource_naming.container.dashboard_anonymous_naming_anonymous_container_1"},
-							HclType:         "dashboard",
+							//HclType:         "dashboard",
 						},
 					},
 					DashboardContainers: map[string]*modconfig.DashboardContainer{
@@ -1205,7 +1241,7 @@ func init() {
 				Require:     require,
 				Description: toStringPointer("This mod contains global inputs"),
 				Title:       toStringPointer("global dashboard inputs"),
-				ResourceMaps: &modconfig.ModResources{
+				ResourceMaps: &modconfig.ResourceMaps{
 					Dashboards: map[string]*modconfig.Dashboard{
 						"global_dashboard_inputs.dashboard.global_dashboard_inputs": {
 							FullName:        "global_dashboard_inputs.dashboard.global_dashboard_inputs",
@@ -1254,7 +1290,7 @@ func init() {
 				Require:     require,
 				Description: toStringPointer("This mod contains a dashboard with all possible child resources"),
 				Title:       toStringPointer("dashboard with all possible child resources"),
-				ResourceMaps: &modconfig.ModResources{
+				ResourceMaps: &modconfig.ResourceMaps{
 					Dashboards: map[string]*modconfig.Dashboard{
 						"dashboard_with_children.dashboard.dashboard_with_child_res": {
 							ShortName:       "dashboard_with_child_res",
@@ -1266,12 +1302,13 @@ func init() {
 								"dashboard_with_children.chart.c1",
 								"dashboard_with_children.card.crd1",
 								"dashboard_with_children.flow.f1",
+								"dashboard_with_children.graph.g1",
 								"dashboard_with_children.hierarchy.h1",
 								"dashboard_with_children.image.i1",
 								"dashboard_with_children.input.ip1",
 								"dashboard_with_children.table.t1",
 								"dashboard_with_children.text.txt1"},
-							HclType: "dashboard",
+							//HclType: "dashboard",
 						},
 					},
 					DashboardContainers: map[string]*modconfig.DashboardContainer{
@@ -1310,6 +1347,15 @@ func init() {
 							Type:            toStringPointer("sankey"),
 						},
 					},
+					DashboardGraphs: map[string]*modconfig.DashboardGraph{
+						"dashboard_with_children.graph.g1": {
+							FullName:        "dashboard_with_children.graph.g1",
+							ShortName:       "g1",
+							UnqualifiedName: "graph.g1",
+							Title:           toStringPointer("example graph"),
+							Type:            toStringPointer("graph"),
+						},
+					},
 					DashboardHierarchies: map[string]*modconfig.DashboardHierarchy{
 						"dashboard_with_children.hierarchy.h1": {
 							FullName:        "dashboard_with_children.hierarchy.h1",
@@ -1403,7 +1449,7 @@ func executeLoadTest(t *testing.T, name string, test loadModTest, wd string) {
 		t.Errorf("failed to build absolute config filepath from %s", test.source)
 	}
 
-	var runCtx = parse.NewRunContext(
+	var parseCtx = parse.NewModParseContext(
 		nil,
 		modPath,
 		parse.CreatePseudoResources|parse.CreateDefaultMod,
@@ -1425,7 +1471,7 @@ func executeLoadTest(t *testing.T, name string, test loadModTest, wd string) {
 		}
 	}()
 
-	actualMod, err := LoadMod(modPath, runCtx)
+	actualMod, err := LoadMod(modPath, parseCtx)
 	if err != nil {
 		if test.expected != "ERROR" {
 			t.Errorf(`Test: '%s'' FAILED : unexpected error %v`, name, err)
@@ -1527,7 +1573,7 @@ func TestLoadModResourceNames(t *testing.T) {
 	for name, test := range testCasesLoadResourceNames {
 
 		modPath, _ := filepath.Abs(test.source)
-		var runCtx = parse.NewRunContext(
+		var parseCtx = parse.NewModParseContext(
 			nil,
 			modPath,
 			parse.CreatePseudoResources|parse.CreateDefaultMod,
@@ -1536,8 +1582,8 @@ func TestLoadModResourceNames(t *testing.T) {
 				Exclude: []string{fmt.Sprintf("**/%s*", filepaths.WorkspaceDataDir)},
 				Flags:   filehelpers.Files,
 			})
-		LoadMod(modPath, runCtx)
-		names, err := LoadModResourceNames(modPath, runCtx)
+		LoadMod(modPath, parseCtx)
+		names, err := LoadModResourceNames(modPath, parseCtx)
 
 		if err != nil {
 			if test.expected != "ERROR" {
