@@ -52,7 +52,7 @@ type Control struct {
 	parents []ModTreeItem
 }
 
-func NewControl(block *hcl.Block, mod *Mod, shortName string) *Control {
+func NewControl(block *hcl.Block, mod *Mod, shortName string) HclResource {
 	control := &Control{
 		ShortName:       shortName,
 		FullName:        fmt.Sprintf("%s.control.%s", mod.ShortName, shortName),
@@ -183,7 +183,7 @@ func (c *Control) GetParents() []ModTreeItem {
 	return c.parents
 }
 
-// GetTitle implements ModTreeItem
+// GetTitle implements HclResource
 func (c *Control) GetTitle() string {
 	return typehelpers.SafeString(c.Title)
 }
@@ -193,7 +193,7 @@ func (c *Control) GetDescription() string {
 	return typehelpers.SafeString(c.Description)
 }
 
-// GetTags implements ModTreeItem
+// GetTags implements HclResource
 func (c *Control) GetTags() map[string]string {
 	if c.Tags != nil {
 		return c.Tags
@@ -242,7 +242,7 @@ func (c *Control) CtyValue() (cty.Value, error) {
 }
 
 // OnDecoded implements HclResource
-func (c *Control) OnDecoded(block *hcl.Block, resourceMapProvider ModResourcesProvider) hcl.Diagnostics {
+func (c *Control) OnDecoded(block *hcl.Block, resourceMapProvider ResourceMapsProvider) hcl.Diagnostics {
 	c.setBaseProperties(resourceMapProvider)
 	// verify the control has either a query or a sql attribute
 	if c.Query == nil && c.SQL == nil {
@@ -256,17 +256,17 @@ func (c *Control) OnDecoded(block *hcl.Block, resourceMapProvider ModResourcesPr
 	return nil
 }
 
-// AddReference implements HclResource
+// AddReference implements ResourceWithMetadata
 func (c *Control) AddReference(ref *ResourceReference) {
 	c.References = append(c.References, ref)
 }
 
-// GetReferences implements HclResource
+// GetReferences implements ResourceWithMetadata
 func (c *Control) GetReferences() []*ResourceReference {
 	return c.References
 }
 
-// GetMod implements HclResource
+// GetMod implements ModTreeItem
 func (c *Control) GetMod() *Mod {
 	return c.Mod
 }
@@ -276,6 +276,11 @@ func (c *Control) GetDeclRange() *hcl.Range {
 	return &c.DeclRange
 }
 
+// BlockType implements HclResource
+func (*Control) BlockType() string {
+	return BlockTypeControl
+}
+
 // GetParams implements QueryProvider
 func (c *Control) GetParams() []*ParamDef {
 	return c.Params
@@ -334,7 +339,7 @@ func (c *Control) GetDisplay() string {
 	return ""
 }
 
-// GetDocumentation implements DashboardLeafNode
+// GetDocumentation implements DashboardLeafNode, ModTreeItem
 func (c *Control) GetDocumentation() string {
 	return typehelpers.SafeString(c.Documentation)
 }
@@ -386,7 +391,7 @@ func (c *Control) Diff(other *Control) *DashboardTreeItemDiffs {
 	return res
 }
 
-func (c *Control) setBaseProperties(resourceMapProvider ModResourcesProvider) {
+func (c *Control) setBaseProperties(resourceMapProvider ResourceMapsProvider) {
 	// not all base properties are stored in the evalContext
 	// (e.g. resource metadata and runtime dependencies are not stores)
 	//  so resolve base from the resource map provider (which is the RunContext)
@@ -414,7 +419,7 @@ func (c *Control) setBaseProperties(resourceMapProvider ModResourcesProvider) {
 	if c.SQL == nil {
 		c.SQL = c.Base.SQL
 	}
-	c.Tags = utils.MergeStringMaps(c.Tags, c.Base.Tags)
+	c.Tags = utils.MergeMaps(c.Tags, c.Base.Tags)
 	if c.Title == nil {
 		c.Title = c.Base.Title
 	}
