@@ -38,7 +38,7 @@ type DashboardContainer struct {
 	runtimeDependencyGraph *topsort.Graph
 }
 
-func NewDashboardContainer(block *hcl.Block, mod *Mod, shortName string) *DashboardContainer {
+func NewDashboardContainer(block *hcl.Block, mod *Mod, shortName string) HclResource {
 	c := &DashboardContainer{
 		ShortName:       shortName,
 		FullName:        fmt.Sprintf("%s.%s.%s", mod.ShortName, block.Type, shortName),
@@ -67,7 +67,7 @@ func (c *DashboardContainer) Name() string {
 }
 
 // OnDecoded implements HclResource
-func (c *DashboardContainer) OnDecoded(block *hcl.Block, resourceMapProvider ModResourcesProvider) hcl.Diagnostics {
+func (c *DashboardContainer) OnDecoded(block *hcl.Block, resourceMapProvider ResourceMapsProvider) hcl.Diagnostics {
 	c.ChildNames = make([]string, len(c.children))
 	for i, child := range c.children {
 		c.ChildNames[i] = child.Name()
@@ -75,17 +75,17 @@ func (c *DashboardContainer) OnDecoded(block *hcl.Block, resourceMapProvider Mod
 	return nil
 }
 
-// AddReference implements HclResource
+// AddReference implements ResourceWithMetadata
 func (c *DashboardContainer) AddReference(ref *ResourceReference) {
 	c.References = append(c.References, ref)
 }
 
-// GetReferences implements HclResource
+// GetReferences implements ResourceWithMetadata
 func (c *DashboardContainer) GetReferences() []*ResourceReference {
 	return c.References
 }
 
-// GetMod implements HclResource
+// GetMod implements ModTreeItem
 func (c *DashboardContainer) GetMod() *Mod {
 	return c.Mod
 }
@@ -95,6 +95,11 @@ func (c *DashboardContainer) GetDeclRange() *hcl.Range {
 	return &c.DeclRange
 }
 
+// BlockType implements HclResource
+func (*DashboardContainer) BlockType() string {
+	return BlockTypeContainer
+}
+
 // AddParent implements ModTreeItem
 func (c *DashboardContainer) AddParent(parent ModTreeItem) error {
 	c.parents = append(c.parents, parent)
@@ -112,7 +117,7 @@ func (c *DashboardContainer) GetChildren() []ModTreeItem {
 	return c.children
 }
 
-// GetTitle implements ModTreeItem
+// GetTitle implements HclResource
 func (c *DashboardContainer) GetTitle() string {
 	return typehelpers.SafeString(c.Title)
 }
@@ -122,9 +127,9 @@ func (c *DashboardContainer) GetDescription() string {
 	return ""
 }
 
-// GetTags implements ModTreeItem
+// GetTags implements HclResource
 func (c *DashboardContainer) GetTags() map[string]string {
-	return nil
+	return map[string]string{}
 }
 
 // GetPaths implements ModTreeItem
@@ -136,6 +141,11 @@ func (c *DashboardContainer) GetPaths() []NodePath {
 	return c.Paths
 }
 
+// GetDocumentation implement ModTreeItem
+func (*DashboardContainer) GetDocumentation() string {
+	return ""
+}
+
 // SetPaths implements ModTreeItem
 func (c *DashboardContainer) SetPaths() {
 	for _, parent := range c.parents {
