@@ -1,134 +1,114 @@
 package modconfig
 
 import (
+	"fmt"
 	"github.com/hashicorp/hcl/v2"
-	"github.com/turbot/go-kit/helpers"
+	"github.com/turbot/steampipe/pkg/utils"
 )
 
-// ModResources is a struct containing maps of all mod resource types
+// ResourceMaps is a struct containing maps of all mod resource types
 // This is provided to avoid db needing to reference workspace package
-type ModResources struct {
+type ResourceMaps struct {
 	// the parent mod
 	Mod *Mod
+
 	// all mods (including deps)
-	Mods                  map[string]*Mod
-	Queries               map[string]*Query
-	Controls              map[string]*Control
 	Benchmarks            map[string]*Benchmark
-	Variables             map[string]*Variable
+	Controls              map[string]*Control
 	Dashboards            map[string]*Dashboard
-	DashboardContainers   map[string]*DashboardContainer
+	DashboardCategories   map[string]*DashboardCategory
 	DashboardCards        map[string]*DashboardCard
 	DashboardCharts       map[string]*DashboardChart
+	DashboardContainers   map[string]*DashboardContainer
+	DashboardEdges        map[string]*DashboardEdge
 	DashboardFlows        map[string]*DashboardFlow
+	DashboardGraphs       map[string]*DashboardGraph
 	DashboardHierarchies  map[string]*DashboardHierarchy
 	DashboardImages       map[string]*DashboardImage
 	DashboardInputs       map[string]map[string]*DashboardInput
-	GlobalDashboardInputs map[string]*DashboardInput
 	DashboardTables       map[string]*DashboardTable
 	DashboardTexts        map[string]*DashboardText
+	DashboardNodes        map[string]*DashboardNode
+	GlobalDashboardInputs map[string]*DashboardInput
+	Locals                map[string]*Local
+	Mods                  map[string]*Mod
+	Queries               map[string]*Query
 	References            map[string]*ResourceReference
-
-	Locals          map[string]*Local
-	LocalQueries    map[string]*Query
-	LocalControls   map[string]*Control
-	LocalBenchmarks map[string]*Benchmark
+	// map of snapshot paths, keyed by snapshot name
+	Snapshots map[string]string
+	Variables map[string]*Variable
 }
 
-func NewWorkspaceResourceMaps(mod *Mod) *ModResources {
-	return &ModResources{
-		Mod:                   mod,
-		Mods:                  map[string]*Mod{mod.Name(): mod},
-		Queries:               make(map[string]*Query),
+func NewModResources(mod *Mod) *ResourceMaps {
+	res := emptyModResources()
+	res.Mod = mod
+	res.Mods[mod.Name()] = mod
+	return res
+}
+
+func NewSourceSnapshotModResources(snapshotPaths []string) *ResourceMaps {
+	res := emptyModResources()
+	res.AddSnapshots(snapshotPaths)
+	return res
+}
+
+func emptyModResources() *ResourceMaps {
+	return &ResourceMaps{
 		Controls:              make(map[string]*Control),
 		Benchmarks:            make(map[string]*Benchmark),
-		Locals:                make(map[string]*Local),
-		Variables:             make(map[string]*Variable),
 		Dashboards:            make(map[string]*Dashboard),
-		DashboardContainers:   make(map[string]*DashboardContainer),
 		DashboardCards:        make(map[string]*DashboardCard),
 		DashboardCharts:       make(map[string]*DashboardChart),
+		DashboardContainers:   make(map[string]*DashboardContainer),
+		DashboardEdges:        make(map[string]*DashboardEdge),
 		DashboardFlows:        make(map[string]*DashboardFlow),
+		DashboardGraphs:       make(map[string]*DashboardGraph),
 		DashboardHierarchies:  make(map[string]*DashboardHierarchy),
 		DashboardImages:       make(map[string]*DashboardImage),
 		DashboardInputs:       make(map[string]map[string]*DashboardInput),
-		GlobalDashboardInputs: make(map[string]*DashboardInput),
 		DashboardTables:       make(map[string]*DashboardTable),
 		DashboardTexts:        make(map[string]*DashboardText),
+		DashboardNodes:        make(map[string]*DashboardNode),
+		DashboardCategories:   make(map[string]*DashboardCategory),
+		GlobalDashboardInputs: make(map[string]*DashboardInput),
+		Locals:                make(map[string]*Local),
+		Mods:                  make(map[string]*Mod),
+		Queries:               make(map[string]*Query),
 		References:            make(map[string]*ResourceReference),
-		LocalQueries:          make(map[string]*Query),
-		LocalControls:         make(map[string]*Control),
-		LocalBenchmarks:       make(map[string]*Benchmark),
+		Snapshots:             make(map[string]string),
+		Variables:             make(map[string]*Variable),
 	}
 }
 
-func CreateWorkspaceResourceMapForQueries(queryProviders []QueryProvider, mod *Mod) *ModResources {
-	res := NewWorkspaceResourceMaps(mod)
+// ModResourcesForQueries creates a ResourceMaps object containing just the specified queries
+// This is used to just create necessary prepared statements when executing batch queries
+func ModResourcesForQueries(queryProviders []QueryProvider, mod *Mod) *ResourceMaps {
+	res := NewModResources(mod)
 	for _, p := range queryProviders {
 		res.addControlOrQuery(p)
 	}
 	return res
 }
 
-func (m *ModResources) QueryProviders() []QueryProvider {
-	numDashboardInputs := 0
-	for _, inputs := range m.DashboardInputs {
-		numDashboardInputs += len(inputs)
-	}
-	res := make([]QueryProvider,
-		len(m.Queries)+
-			len(m.Controls)+
-			len(m.DashboardCards)+
-			len(m.DashboardCharts)+
-			len(m.DashboardFlows)+
-			len(m.DashboardHierarchies)+
-			numDashboardInputs+
-			len(m.GlobalDashboardInputs)+
-			len(m.DashboardTables))
-
+// QueryProviders returns a slice of all QueryProviders
+func (m *ResourceMaps) QueryProviders() []QueryProvider {
+	res := make([]QueryProvider, m.queryProviderCount())
 	idx := 0
-	for _, p := range m.Queries {
-		res[idx] = p
-		idx++
-	}
-	for _, p := range m.Controls {
-		res[idx] = p
-		idx++
-	}
-	for _, p := range m.DashboardCards {
-		res[idx] = p
-		idx++
-	}
-	for _, p := range m.DashboardCharts {
-		res[idx] = p
-		idx++
-	}
-	for _, p := range m.DashboardFlows {
-		res[idx] = p
-		idx++
-	}
-	for _, p := range m.DashboardHierarchies {
-		res[idx] = p
-		idx++
-	}
-	for _, inputsForDashboard := range m.DashboardInputs {
-		for _, p := range inputsForDashboard {
-			res[idx] = p
+	f := func(item HclResource) (bool, error) {
+		if queryProvider, ok := item.(QueryProvider); ok {
+			res[idx] = queryProvider
 			idx++
 		}
+		return true, nil
 	}
-	for _, p := range m.GlobalDashboardInputs {
-		res[idx] = p
-		idx++
-	}
-	for _, p := range m.DashboardTables {
-		res[idx] = p
-		idx++
-	}
+
+	m.WalkResources(f)
+
 	return res
 }
 
-func (m *ModResources) Equals(other *ModResources) bool {
+func (m *ResourceMaps) Equals(other *ResourceMaps) bool {
+	//TODO use cmp.Equals or similar
 	if other == nil {
 		return false
 	}
@@ -250,6 +230,19 @@ func (m *ModResources) Equals(other *ModResources) bool {
 		}
 	}
 
+	for name, flows := range m.DashboardGraphs {
+		if otherFlow, ok := other.DashboardGraphs[name]; !ok {
+			return false
+		} else if !flows.Equals(otherFlow) {
+			return false
+		}
+	}
+	for name := range other.DashboardGraphs {
+		if _, ok := m.DashboardGraphs[name]; !ok {
+			return false
+		}
+	}
+
 	for name, hierarchies := range m.DashboardHierarchies {
 		if otherHierarchy, ok := other.DashboardHierarchies[name]; !ok {
 			return false
@@ -257,8 +250,20 @@ func (m *ModResources) Equals(other *ModResources) bool {
 			return false
 		}
 	}
-	for name := range other.DashboardHierarchies {
-		if _, ok := m.DashboardHierarchies[name]; !ok {
+
+	for name := range other.DashboardNodes {
+		if _, ok := m.DashboardNodes[name]; !ok {
+			return false
+		}
+	}
+
+	for name := range other.DashboardEdges {
+		if _, ok := m.DashboardEdges[name]; !ok {
+			return false
+		}
+	}
+	for name := range other.DashboardCategories {
+		if _, ok := m.DashboardCategories[name]; !ok {
 			return false
 		}
 	}
@@ -322,6 +327,13 @@ func (m *ModResources) Equals(other *ModResources) bool {
 			return false
 		}
 	}
+	for name, Categorys := range m.DashboardCategories {
+		if otherCategory, ok := other.DashboardCategories[name]; !ok {
+			return false
+		} else if !Categorys.Equals(otherCategory) {
+			return false
+		}
+	}
 	for name := range other.DashboardTables {
 		if _, ok := m.DashboardTables[name]; !ok {
 			return false
@@ -363,13 +375,12 @@ func (m *ModResources) Equals(other *ModResources) bool {
 	return true
 }
 
-func (m *ModResources) PopulateReferences() {
+func (m *ResourceMaps) PopulateReferences() {
 	m.References = make(map[string]*ResourceReference)
 
 	resourceFunc := func(resource HclResource) (bool, error) {
-		parsedName, _ := ParseResourceName(resource.Name())
-		if helpers.StringSliceContains(ReferenceBlocks, parsedName.ItemType) {
-			for _, ref := range resource.GetReferences() {
+		if resourceWithMetadata, ok := resource.(ResourceWithMetadata); ok {
+			for _, ref := range resourceWithMetadata.GetReferences() {
 				m.References[ref.String()] = ref
 			}
 		}
@@ -380,7 +391,7 @@ func (m *ModResources) PopulateReferences() {
 	m.WalkResources(resourceFunc)
 }
 
-func (m *ModResources) Empty() bool {
+func (m *ResourceMaps) Empty() bool {
 	return len(m.Mods)+
 		len(m.Queries)+
 		len(m.Controls)+
@@ -391,7 +402,11 @@ func (m *ModResources) Empty() bool {
 		len(m.DashboardCards)+
 		len(m.DashboardCharts)+
 		len(m.DashboardFlows)+
+		len(m.DashboardGraphs)+
 		len(m.DashboardHierarchies)+
+		len(m.DashboardNodes)+
+		len(m.DashboardEdges)+
+		len(m.DashboardCategories)+
 		len(m.DashboardImages)+
 		len(m.DashboardInputs)+
 		len(m.DashboardTables)+
@@ -399,8 +414,8 @@ func (m *ModResources) Empty() bool {
 		len(m.References) == 0
 }
 
-// this is used to create an optimized ModResources containing only the queries which will be run
-func (m *ModResources) addControlOrQuery(provider QueryProvider) {
+// this is used to create an optimized ResourceMaps containing only the queries which will be run
+func (m *ResourceMaps) addControlOrQuery(provider QueryProvider) {
 	switch p := provider.(type) {
 	case *Query:
 		if p != nil {
@@ -415,8 +430,8 @@ func (m *ModResources) addControlOrQuery(provider QueryProvider) {
 
 // WalkResources calls resourceFunc for every resource in the mod
 // if any resourceFunc returns false or an error, return immediately
-func (m *ModResources) WalkResources(resourceFunc func(item HclResource) (bool, error)) error {
-	for _, r := range m.Queries {
+func (m *ResourceMaps) WalkResources(resourceFunc func(item HclResource) (bool, error)) error {
+	for _, r := range m.Benchmarks {
 		if continueWalking, err := resourceFunc(r); err != nil || !continueWalking {
 			return err
 		}
@@ -426,36 +441,46 @@ func (m *ModResources) WalkResources(resourceFunc func(item HclResource) (bool,
 			return err
 		}
 	}
-	for _, r := range m.Benchmarks {
-		if continueWalking, err := resourceFunc(r); err != nil || !continueWalking {
-			return err
-		}
-	}
 	for _, r := range m.Dashboards {
 		if continueWalking, err := resourceFunc(r); err != nil || !continueWalking {
 			return err
 		}
 	}
-	for _, r := range m.DashboardContainers {
-		if continueWalking, err := resourceFunc(r); err != nil || !continueWalking {
-			return err
-		}
-	}
 	for _, r := range m.DashboardCards {
 		if continueWalking, err := resourceFunc(r); err != nil || !continueWalking {
 			return err
 		}
 	}
+	for _, r := range m.DashboardCategories {
+		if continueWalking, err := resourceFunc(r); err != nil || !continueWalking {
+			return err
+		}
+	}
 	for _, r := range m.DashboardCharts {
 		if continueWalking, err := resourceFunc(r); err != nil || !continueWalking {
 			return err
 		}
 	}
+	for _, r := range m.DashboardContainers {
+		if continueWalking, err := resourceFunc(r); err != nil || !continueWalking {
+			return err
+		}
+	}
+	for _, r := range m.DashboardEdges {
+		if continueWalking, err := resourceFunc(r); err != nil || !continueWalking {
+			return err
+		}
+	}
 	for _, r := range m.DashboardFlows {
 		if continueWalking, err := resourceFunc(r); err != nil || !continueWalking {
 			return err
 		}
 	}
+	for _, r := range m.DashboardGraphs {
+		if continueWalking, err := resourceFunc(r); err != nil || !continueWalking {
+			return err
+		}
+	}
 	for _, r := range m.DashboardHierarchies {
 		if continueWalking, err := resourceFunc(r); err != nil || !continueWalking {
 			return err
@@ -466,11 +491,6 @@ func (m *ModResources) WalkResources(resourceFunc func(item HclResource) (bool,
 			return err
 		}
 	}
-	for _, r := range m.GlobalDashboardInputs {
-		if continueWalking, err := resourceFunc(r); err != nil || !continueWalking {
-			return err
-		}
-	}
 	for _, inputsForDashboard := range m.DashboardInputs {
 		for _, r := range inputsForDashboard {
 			if continueWalking, err := resourceFunc(r); err != nil || !continueWalking {
@@ -478,6 +498,11 @@ func (m *ModResources) WalkResources(resourceFunc func(item HclResource) (bool,
 			}
 		}
 	}
+	for _, r := range m.DashboardNodes {
+		if continueWalking, err := resourceFunc(r); err != nil || !continueWalking {
+			return err
+		}
+	}
 	for _, r := range m.DashboardTables {
 		if continueWalking, err := resourceFunc(r); err != nil || !continueWalking {
 			return err
@@ -488,7 +513,7 @@ func (m *ModResources) WalkResources(resourceFunc func(item HclResource) (bool,
 			return err
 		}
 	}
-	for _, r := range m.Variables {
+	for _, r := range m.GlobalDashboardInputs {
 		if continueWalking, err := resourceFunc(r); err != nil || !continueWalking {
 			return err
 		}
@@ -498,10 +523,22 @@ func (m *ModResources) WalkResources(resourceFunc func(item HclResource) (bool,
 			return err
 		}
 	}
+	for _, r := range m.Queries {
+		if continueWalking, err := resourceFunc(r); err != nil || !continueWalking {
+			return err
+		}
+	}
+	// we cannot walk source snapshots as they are not a HclResource
+	for _, r := range m.Variables {
+		if continueWalking, err := resourceFunc(r); err != nil || !continueWalking {
+			return err
+		}
+	}
+
 	return nil
 }
 
-func (m *ModResources) AddResource(item HclResource) hcl.Diagnostics {
+func (m *ResourceMaps) AddResource(item HclResource) hcl.Diagnostics {
 	var diags hcl.Diagnostics
 	switch r := item.(type) {
 	case *Query:
@@ -511,8 +548,6 @@ func (m *ModResources) AddResource(item HclResource) hcl.Diagnostics {
 			break
 		}
 		m.Queries[name] = r
-		// also add to LocalQueries
-		m.LocalQueries[r.GetUnqualifiedName()] = r
 
 	case *Control:
 		name := r.Name()
@@ -521,8 +556,6 @@ func (m *ModResources) AddResource(item HclResource) hcl.Diagnostics {
 			break
 		}
 		m.Controls[name] = r
-		// also add to LocalControls
-		m.LocalControls[r.GetUnqualifiedName()] = r
 
 	case *Benchmark:
 		name := r.Name()
@@ -531,8 +564,6 @@ func (m *ModResources) AddResource(item HclResource) hcl.Diagnostics {
 			break
 		}
 		m.Benchmarks[name] = r
-		// also add to LocalBenchmarks
-		m.LocalBenchmarks[r.GetUnqualifiedName()] = r
 
 	case *Dashboard:
 		name := r.Name()
@@ -575,6 +606,14 @@ func (m *ModResources) AddResource(item HclResource) hcl.Diagnostics {
 		}
 		m.DashboardFlows[name] = r
 
+	case *DashboardGraph:
+		name := r.Name()
+		if existing, ok := m.DashboardGraphs[name]; ok {
+			diags = append(diags, checkForDuplicate(existing, item)...)
+			break
+		}
+		m.DashboardGraphs[name] = r
+
 	case *DashboardHierarchy:
 		name := r.Name()
 		if existing, ok := m.DashboardHierarchies[name]; ok {
@@ -583,6 +622,29 @@ func (m *ModResources) AddResource(item HclResource) hcl.Diagnostics {
 		}
 		m.DashboardHierarchies[name] = r
 
+	case *DashboardNode:
+		name := r.Name()
+		if existing, ok := m.DashboardNodes[name]; ok {
+			diags = append(diags, checkForDuplicate(existing, item)...)
+			break
+		}
+		m.DashboardNodes[name] = r
+
+	case *DashboardEdge:
+		name := r.Name()
+		if existing, ok := m.DashboardEdges[name]; ok {
+			diags = append(diags, checkForDuplicate(existing, item)...)
+			break
+		}
+		m.DashboardEdges[name] = r
+	case *DashboardCategory:
+		name := r.Name()
+		if existing, ok := m.DashboardCategories[name]; ok {
+			diags = append(diags, checkForDuplicate(existing, item)...)
+			break
+		}
+		m.DashboardCategories[name] = r
+
 	case *DashboardImage:
 		name := r.Name()
 		if existing, ok := m.DashboardImages[name]; ok {
@@ -649,42 +711,23 @@ func (m *ModResources) AddResource(item HclResource) hcl.Diagnostics {
 	return diags
 }
 
-func (m *ModResources) Merge(others []*ModResources) *ModResources {
-	res := NewWorkspaceResourceMaps(m.Mod)
-	sourceMaps := append([]*ModResources{m}, others...)
+func (m *ResourceMaps) AddSnapshots(snapshotPaths []string) {
+	for _, snapshotPath := range snapshotPaths {
+		snapshotName := fmt.Sprintf("snapshot.%s", utils.FilenameNoExtension(snapshotPath))
+		m.Snapshots[snapshotName] = snapshotPath
+	}
+}
 
-	// take local resources from ourselves
-	for k, v := range m.LocalQueries {
-		res.LocalQueries[k] = v
-	}
-	for k, v := range m.LocalControls {
-		res.LocalControls[k] = v
-	}
-	for k, v := range m.LocalBenchmarks {
-		res.LocalBenchmarks[k] = v
-	}
+func (m *ResourceMaps) Merge(others []*ResourceMaps) *ResourceMaps {
+	res := NewModResources(m.Mod)
+	sourceMaps := append([]*ResourceMaps{m}, others...)
 
 	for _, source := range sourceMaps {
-		for k, v := range source.Mods {
-			res.Mods[k] = v
-		}
-		for k, v := range source.Queries {
-			res.Queries[k] = v
-		}
-		for k, v := range source.Controls {
-			res.Controls[k] = v
-		}
 		for k, v := range source.Benchmarks {
 			res.Benchmarks[k] = v
 		}
-		for k, v := range source.Locals {
-			res.Locals[k] = v
-		}
-		for k, v := range source.Variables {
-			// NOTE: only include variables from root mod  - we add in the others separately
-			if v.Mod.FullName == m.Mod.FullName {
-				res.Variables[k] = v
-			}
+		for k, v := range source.Controls {
+			res.Controls[k] = v
 		}
 		for k, v := range source.Dashboards {
 			res.Dashboards[k] = v
@@ -695,31 +738,84 @@ func (m *ModResources) Merge(others []*ModResources) *ModResources {
 		for k, v := range source.DashboardCards {
 			res.DashboardCards[k] = v
 		}
+		for k, v := range source.DashboardCategories {
+			res.DashboardCategories[k] = v
+		}
 		for k, v := range source.DashboardCharts {
 			res.DashboardCharts[k] = v
 		}
+		for k, v := range source.DashboardEdges {
+			res.DashboardEdges[k] = v
+		}
 		for k, v := range source.DashboardFlows {
 			res.DashboardFlows[k] = v
 		}
+		for k, v := range source.DashboardGraphs {
+			res.DashboardGraphs[k] = v
+		}
 		for k, v := range source.DashboardHierarchies {
 			res.DashboardHierarchies[k] = v
 		}
+		for k, v := range source.DashboardNodes {
+			res.DashboardNodes[k] = v
+		}
 		for k, v := range source.DashboardImages {
 			res.DashboardImages[k] = v
 		}
 		for k, v := range source.DashboardInputs {
 			res.DashboardInputs[k] = v
 		}
-		for k, v := range source.GlobalDashboardInputs {
-			res.GlobalDashboardInputs[k] = v
-		}
 		for k, v := range source.DashboardTables {
 			res.DashboardTables[k] = v
 		}
 		for k, v := range source.DashboardTexts {
 			res.DashboardTexts[k] = v
 		}
+		for k, v := range source.GlobalDashboardInputs {
+			res.GlobalDashboardInputs[k] = v
+		}
+		for k, v := range source.Locals {
+			res.Locals[k] = v
+		}
+		for k, v := range source.Mods {
+			res.Mods[k] = v
+		}
+		for k, v := range source.Queries {
+			res.Queries[k] = v
+		}
+		for k, v := range source.Snapshots {
+			res.Snapshots[k] = v
+		}
+		for k, v := range source.Variables {
+			// NOTE: only include variables from root mod  - we add in the others separately
+			if v.Mod.FullName == m.Mod.FullName {
+				res.Variables[k] = v
+			}
+		}
 	}
 
 	return res
 }
+
+func (m *ResourceMaps) queryProviderCount() int {
+	numDashboardInputs := 0
+	for _, inputs := range m.DashboardInputs {
+		numDashboardInputs += len(inputs)
+	}
+
+	numItems :=
+		len(m.Controls) +
+			len(m.DashboardCards) +
+			len(m.DashboardCharts) +
+			len(m.DashboardEdges) +
+			len(m.DashboardFlows) +
+			len(m.DashboardGraphs) +
+			len(m.DashboardHierarchies) +
+			len(m.DashboardImages) +
+			numDashboardInputs +
+			len(m.DashboardNodes) +
+			len(m.DashboardTables) +
+			len(m.GlobalDashboardInputs) +
+			len(m.Queries)
+	return numItems
+}
