@@ -34,7 +34,9 @@ func ParseResourceName(fullName string) (res *ParsedResourceName, err error) {
 	default:
 		err = fmt.Errorf("invalid name '%s' passed to ParseResourceName", fullName)
 	}
-
+	if !IsValidResourceItemType(res.ItemType) {
+		err = fmt.Errorf("invalid name '%s' passed to ParseResourceName", fullName)
+	}
 	return
 }
 
@@ -42,6 +44,20 @@ func (p *ParsedResourceName) ToResourceName() string {
 	return BuildModResourceName(p.ItemType, p.Name)
 }
 
+func (p *ParsedResourceName) ToFullName() string {
+	return BuildFullResourceName(p.Mod, p.ItemType, p.Name)
+}
+func (p *ParsedResourceName) ToFullNameWithMod(mod string) string {
+	if p.Mod != "" {
+		return p.ToFullName()
+	}
+	return BuildFullResourceName(mod, p.ItemType, p.Name)
+}
+
+func BuildFullResourceName(mod, blockType, name string) string {
+	return fmt.Sprintf("%s.%s.%s", mod, blockType, name)
+}
+
 // UnqualifiedResourceName removes the mod prefix from the given name
 func UnqualifiedResourceName(fullName string) string {
 	parts := strings.Split(fullName, ".")
@@ -53,6 +69,6 @@ func UnqualifiedResourceName(fullName string) string {
 	}
 }
 
-func BuildModResourceName(blockType string, name string) string {
+func BuildModResourceName(blockType, name string) string {
 	return fmt.Sprintf("%s.%s", blockType, name)
 }
