@@ -155,7 +155,7 @@ func ParseModDefinition(modPath string) (*modconfig.Mod, error) {
 
 // ParseMod parses all source hcl files for the mod path and associated resources, and returns the mod object
 // NOTE: the mod definition has already been parsed (or a default created) and is in opts.RunCtx.RootMod
-func ParseMod(modPath string, fileData map[string][]byte, pseudoResources []modconfig.MappableResource, runCtx *RunContext) (*modconfig.Mod, error) {
+func ParseMod(modPath string, fileData map[string][]byte, pseudoResources []modconfig.MappableResource, parseCtx *ModParseContext) (*modconfig.Mod, error) {
 	body, diags := ParseHclFiles(fileData)
 	if diags.HasErrors() {
 		return nil, plugin.DiagsToError("Failed to load all mod source files", diags)
@@ -167,9 +167,9 @@ func ParseMod(modPath string, fileData map[string][]byte, pseudoResources []modc
 		return nil, plugin.DiagsToError("Failed to load mod", diags)
 	}
 
-	mod := runCtx.CurrentMod
+	mod := parseCtx.CurrentMod
 	if mod == nil {
-		return nil, fmt.Errorf("ParseMod called with no Current Mod set in RunContext")
+		return nil, fmt.Errorf("ParseMod called with no Current Mod set in ModParseContext")
 	}
 	// get names of all resources defined in hcl which may also be created as pseudo resources
 	hclResources, err := loadMappableResourceNames(modPath, content)
@@ -178,7 +178,7 @@ func ParseMod(modPath string, fileData map[string][]byte, pseudoResources []modc
 	}
 
 	// if variables were passed in runcontext, add to the mod
-	for _, v := range runCtx.Variables {
+	for _, v := range parseCtx.Variables {
 		if diags = mod.AddResource(v); diags.HasErrors() {
 			return nil, plugin.DiagsToError("Failed to add resource to mod", diags)
 		}
@@ -188,11 +188,11 @@ func ParseMod(modPath string, fileData map[string][]byte, pseudoResources []modc
 	addPseudoResourcesToMod(pseudoResources, hclResources, mod)
 
 	// add the parsed content to the run context
-	runCtx.SetDecodeContent(content, fileData)
+	parseCtx.SetDecodeContent(content, fileData)
 
 	// add the mod to the run context
 	// - this it to ensure all pseudo resources get added and build the eval context with the variables we just added
-	if diags = runCtx.AddMod(mod); diags.HasErrors() {
+	if diags = parseCtx.AddMod(mod); diags.HasErrors() {
 		return nil, plugin.DiagsToError("Failed to add mod to run context", diags)
 	}
 
@@ -200,20 +200,20 @@ func ParseMod(modPath string, fileData map[string][]byte, pseudoResources []modc
 	// continue decoding as long as the number of unresolved blocks decreases
 	prevUnresolvedBlocks := 0
 	for attempts := 0; ; attempts++ {
-		diags = decode(runCtx)
+		diags = decode(parseCtx)
 		if diags.HasErrors() {
 			return nil, plugin.DiagsToError("Failed to decode all mod hcl files", diags)
 		}
 
 		// if there are no unresolved blocks, we are done
-		unresolvedBlocks := len(runCtx.UnresolvedBlocks)
+		unresolvedBlocks := len(parseCtx.UnresolvedBlocks)
 		if unresolvedBlocks == 0 {
 			log.Printf("[TRACE] parse complete after %d decode passes", attempts+1)
 			break
 		}
 		// if the number of unresolved blocks has NOT reduced, fail
 		if prevUnresolvedBlocks != 0 && unresolvedBlocks >= prevUnresolvedBlocks {
-			str := runCtx.FormatDependencies()
+			str := parseCtx.FormatDependencies()
 			return nil, fmt.Errorf("failed to resolve mod dependencies after %d attempts\nDependencies:\n%s", attempts+1, str)
 		}
 		// update prevUnresolvedBlocks
@@ -221,7 +221,7 @@ func ParseMod(modPath string, fileData map[string][]byte, pseudoResources []modc
 	}
 
 	// now tell mod to build tree of controls.
-	if err := mod.BuildResourceTree(runCtx.LoadedDependencyMods); err != nil {
+	if err := mod.BuildResourceTree(parseCtx.LoadedDependencyMods); err != nil {
 		return nil, err
 	}
 
