@@ -1,12 +1,8 @@
 package pluginmanager
 
 import (
+	"crypto/md5"
 	"fmt"
-	"github.com/hashicorp/go-hclog"
-	"github.com/hashicorp/go-plugin"
-	sdkgrpc "github.com/turbot/steampipe-plugin-sdk/v4/grpc"
-	sdkproto "github.com/turbot/steampipe-plugin-sdk/v4/grpc/proto"
-	"github.com/turbot/steampipe/pkg/constants"
 	"log"
 	"os"
 	"os/exec"
@@ -16,14 +12,21 @@ import (
 	"sync"
 	"time"
 
+	"github.com/hashicorp/go-hclog"
+	"github.com/hashicorp/go-plugin"
 	"github.com/turbot/go-kit/helpers"
+	sdkgrpc "github.com/turbot/steampipe-plugin-sdk/v4/grpc"
+	sdkproto "github.com/turbot/steampipe-plugin-sdk/v4/grpc/proto"
 	sdkshared "github.com/turbot/steampipe-plugin-sdk/v4/grpc/shared"
+	"github.com/turbot/steampipe/pkg/constants"
+	"github.com/turbot/steampipe/pkg/error_helpers"
 	"github.com/turbot/steampipe/pkg/utils"
 	"github.com/turbot/steampipe/pluginmanager/grpc/proto"
 	pluginshared "github.com/turbot/steampipe/pluginmanager/grpc/shared"
 )
 
 type runningPlugin struct {
+	pluginName  string
 	client      *plugin.Client
 	reattach    *proto.ReattachConfig
 	initialized chan struct{}
@@ -124,13 +127,7 @@ func (m *PluginManager) SetConnectionConfigMap(configMap map[string]*sdkproto.Co
 	m.mut.Lock()
 	defer m.mut.Unlock()
 
-	// TODO add getMapKeys function
-	names := make([]string, len(configMap))
-	idx := 0
-	for name := range configMap {
-		names[idx] = name
-		idx++
-	}
+	names := utils.SortedMapKeys(configMap)
 	log.Printf("[TRACE] SetConnectionConfigMap: %s", strings.Join(names, ","))
 
 	err := m.handleConnectionConfigChanges(configMap)
@@ -182,7 +179,7 @@ func (m *PluginManager) sendUpdateConnectionConfigs(requestMap map[string]*sdkpr
 			errors = append(errors, err)
 		}
 	}
-	return utils.CombineErrors(errors...)
+	return error_helpers.CombineErrors(errors...)
 }
 
 // this mutates requestMap
@@ -450,6 +447,7 @@ func (m *PluginManager) addLoadingPlugin(connectionName string, p *runningPlugin
 	// add a new running plugin to both connectionPluginMap and pluginMap
 	// NOTE: m.mut must be locked before calling this
 	p = &runningPlugin{
+		pluginName:  pluginName,
 		initialized: make(chan struct{}, 1),
 	}
 	m.connectionPluginMap[connectionName] = p
@@ -503,8 +501,8 @@ func (m *PluginManager) setPluginCacheSizeMap() {
 	// get total connection count for this plugin (excluding aggregators)
 	numConnections := m.nonAggregatorConnectionCount()
 
-	log.Printf("[WARN] PluginManager setPluginCacheSizeMap: %d %s.", numConnections, utils.Pluralize("connection", numConnections))
-	log.Printf("[WARN] Total cache size %dMb", maxCacheSizeMb)
+	log.Printf("[TRACE] PluginManager setPluginCacheSizeMap: %d %s.", numConnections, utils.Pluralize("connection", numConnections))
+	log.Printf("[TRACE] Total cache size %dMb", maxCacheSizeMb)
 
 	for plugin, connections := range m.pluginConnectionConfigMap {
 		var size int64 = 0
@@ -537,6 +535,7 @@ func (m *PluginManager) startPlugin(connectionName string) (_ *plugin.Client, _
 	if err != nil {
 		return nil, nil, err
 	}
+	log.Printf("[TRACE] ************ plugin path %s ********************\n", pluginPath)
 
 	// create the plugin map
 	pluginName := connectionConfig.Plugin
@@ -544,12 +543,22 @@ func (m *PluginManager) startPlugin(connectionName string) (_ *plugin.Client, _
 		pluginName: &sdkshared.WrapperPlugin{},
 	}
 
+	utils.LogTime("getting plugin exec hash")
+	pluginChecksum, err := utils.FileMD5Hash(pluginPath)
+	if err != nil {
+		return nil, nil, err
+	}
+	utils.LogTime("got plugin exec hash")
 	cmd := exec.Command(pluginPath)
 	client := plugin.NewClient(&plugin.ClientConfig{
 		HandshakeConfig:  sdkshared.Handshake,
 		Plugins:          pluginMap,
 		Cmd:              cmd,
 		AllowedProtocols: []plugin.Protocol{plugin.ProtocolGRPC},
+		SecureConfig: &plugin.SecureConfig{
+			Checksum: pluginChecksum,
+			Hash:     md5.New(),
+		},
 		// pass our logger to the plugin client to ensure plugin logs end up in logfile
 		Logger: m.logger,
 	})
@@ -610,7 +619,7 @@ func (m *PluginManager) waitForPluginLoad(p *runningPlugin) error {
 		return nil
 
 	case <-time.After(time.Duration(pluginStartTimeoutSecs) * time.Second):
-		return fmt.Errorf("timed out waiting for %s to startup after %d seconds", p.reattach.Plugin, pluginStartTimeoutSecs)
+		return fmt.Errorf("timed out waiting for %s to startup after %d seconds", p.pluginName, pluginStartTimeoutSecs)
 	}
 }
 
