@@ -4,9 +4,11 @@ import merge from "lodash/merge";
 import React, { useEffect, useRef, useState } from "react";
 import ReactEChartsCore from "echarts-for-react/lib/core";
 import set from "lodash/set";
+import useChartThemeColors from "../../../../hooks/useChartThemeColors";
 import useMediaMode from "../../../../hooks/useMediaMode";
 import {
   BarChart,
+  GraphChart,
   LineChart,
   PieChart,
   SankeyChart,
@@ -38,17 +40,20 @@ import {
 } from "../types";
 import { FlowType } from "../../flows/types";
 import { getChartComponent } from "..";
+import { GraphType } from "../../graphs/types";
 import { HierarchyType } from "../../hierarchies/types";
 import { LabelLayout } from "echarts/features";
-import { Theme } from "../../../../hooks/useTheme";
-import { useDashboard } from "../../../../hooks/useDashboard";
-import * as echarts from "echarts/core";
 import { registerComponent } from "../../index";
+import { renderInterpolatedTemplates } from "../../../../utils/template";
+import { useDashboard } from "../../../../hooks/useDashboard";
+import { useNavigate } from "react-router-dom";
+import * as echarts from "echarts/core";
 
 echarts.use([
   BarChart,
   CanvasRenderer,
   DatasetComponent,
+  GraphChart,
   GridComponent,
   LabelLayout,
   LegendComponent,
@@ -621,28 +626,7 @@ const getSeriesForChartType = (
   return series;
 };
 
-const buildChartOptions = (
-  props: ChartProps,
-  theme: Theme,
-  themeWrapperRef: ((instance: null) => void) | React.RefObject<null>
-) => {
-  // We need to get the theme CSS variable values - these are accessible on the theme root element and below in the tree
-  // @ts-ignore
-  const style = window.getComputedStyle(themeWrapperRef);
-  const foreground = style.getPropertyValue("--color-foreground");
-  const foregroundLightest = style.getPropertyValue(
-    "--color-foreground-lightest"
-  );
-  const alert = style.getPropertyValue("--color-alert");
-  const info = style.getPropertyValue("--color-info");
-  const ok = style.getPropertyValue("--color-ok");
-  const themeColors = {
-    foreground,
-    foregroundLightest,
-    alert,
-    info,
-    ok,
-  };
+const buildChartOptions = (props: ChartProps, themeColors: any) => {
   const { dataset, rowSeriesLabels, transform } = buildChartDataset(
     props.data,
     props.properties
@@ -680,10 +664,32 @@ const buildChartOptions = (
 
 interface ChartComponentProps {
   options: EChartsOption;
-  type: ChartType | FlowType | HierarchyType;
+  type: ChartType | FlowType | GraphType | HierarchyType;
 }
 
+const handleClick = async (params: any, navigate) => {
+  const componentType = params.componentType;
+  if (componentType !== "series") {
+    return;
+  }
+  const dataType = params.dataType;
+
+  switch (dataType) {
+    case "node":
+      if (!params.data.href) {
+        return;
+      }
+      const renderedResults = await renderInterpolatedTemplates(
+        { graph_node: params.data.href as string },
+        [params.data]
+      );
+      let rowRenderResult = renderedResults[0];
+      navigate(rowRenderResult.graph_node.result);
+  }
+};
+
 const Chart = ({ options, type }: ChartComponentProps) => {
+  const navigate = useNavigate();
   const chartRef = useRef<ReactEChartsCore>(null);
   const [imageUrl, setImageUrl] = useState<string | null>(null);
   const mediaMode = useMediaMode();
@@ -694,7 +700,7 @@ const Chart = ({ options, type }: ChartComponentProps) => {
     }
 
     const echartInstance = chartRef.current.getEchartsInstance();
-    const dataURL = echartInstance.getDataURL();
+    const dataURL = echartInstance.getDataURL({});
     if (dataURL === imageUrl) {
       return;
     }
@@ -705,6 +711,10 @@ const Chart = ({ options, type }: ChartComponentProps) => {
     return null;
   }
 
+  const eventsDict = {
+    click: (params) => handleClick(params, navigate),
+  };
+
   return (
     <>
       {mediaMode !== "print" && (
@@ -713,6 +723,7 @@ const Chart = ({ options, type }: ChartComponentProps) => {
             ref={chartRef}
             echarts={echarts}
             className="chart-canvas"
+            onEvents={eventsDict}
             option={options}
             notMerge={true}
             lazyUpdate={true}
@@ -732,13 +743,10 @@ const Chart = ({ options, type }: ChartComponentProps) => {
 };
 
 const ChartWrapper = (props: ChartProps) => {
-  const [, setRandomVal] = useState(0);
   const {
-    themeContext: { theme, wrapperRef },
+    themeContext: { wrapperRef },
   } = useDashboard();
-
-  // This is annoying, but unless I force a refresh the theme doesn't stay in sync when you switch
-  useEffect(() => setRandomVal(Math.random()), [theme.name]);
+  const themeColors = useChartThemeColors();
 
   if (!wrapperRef) {
     return null;
@@ -750,7 +758,7 @@ const ChartWrapper = (props: ChartProps) => {
 
   return (
     <Chart
-      options={buildChartOptions(props, theme, wrapperRef)}
+      options={buildChartOptions(props, themeColors)}
       type={props.display_type || "column"}
     />
   );
